--[[--------------------------------------------------------------
Disenchant Crafted Items
by Sunsequent

A button in profession window for fast disenchanting of crafted 
items without the need to target them in bags.

Version 2.2.1
Release Date 25.10.2022
--------------------------------------------------------------]]--

---- Version 2.2.1
-- Ignore Cosmetic items

---- Version 2.2.0
-- Updated for Dragonflight

-- Configuration Parameters
local ignoreSoulbound = true;
local autoLoot = true;

-- Variables
local selectedItemID;
local numberOfItems = 0;
local itemLocations = {};
local disenchantCasting = false;
local itemLocked = false;
local lootOpen = false;
local locationDisenchantStarted;
local previousItemBagSlot;
local autoLootCurrent = 0;

-- Localization
local locale;
local disenchantName;
local tailoringName;
local leatherworkingName;
local blacksmithingName;
local jewelcraftingName;
local engineeringName;
local inscriptionName;
local professionsForButton;
local itemCountText;
local buttonWidth;
local buttonLocX;

SLASH_DISENCHANTCRAFTEDITEMS1, SLASH_DISENCHANTCRAFTEDITEMS2 = "/disenchantcrafteditems", "/dci";

_G["BINDING_HEADER_DISENCHANTCRAFTEDITEMS"] = "Disenchant Crafted Items";

function SlashCmdList.DISENCHANTCRAFTEDITEMS(msg, editbox)
    if msg == "autoloot" then
        if autoLoot then
			autoLoot = false;
			DCI_AutoLootDisenchanted = autoLoot;
			
			print("Disenchanted materials are not autolooted.");
		else
			autoLoot = true;
			DCI_AutoLootDisenchanted = autoLoot;
			
			print("Disenchanted materials are autolooted.");
		end

        return;
    end
	
	if msg == "soulbound" then
        if ignoreSoulbound then
			ignoreSoulbound = false;
			DCI_IgnoreSoulbound = ignoreSoulbound;
			
			print("Soulbound crafted items are included in disenchanting.");
		else
			ignoreSoulbound = true;
			DCI_IgnoreSoulbound = ignoreSoulbound;
			
			print("Soulbound crafted items are excluded from disenchanting.");
		end

        return;
    end
	
	if msg == "status" then
		if not ignoreSoulbound then
			print("Soulbound crafted items are included in disenchanting.");
		else
			print("Soulbound crafted items are excluded from disenchanting.");
		end
		
		if not autoLoot then
			print("Disenchanted materials are not autolooted.");
		else
			print("Disenchanted materials are autolooted.");
		end
		
		return;
	end
	
	print("Disenchant Crafted Items (also /dci) commands:");
	print("/disenchantcrafteditems autoloot");
	print("/disenchantcrafteditems soulbound");
	print("/disenchantcrafteditems status");
end

local function SetLocalization()
	-- Localization:
	-- enUS Items in backpack			Disenchant
	-- deDE Gegenstände im Rucksack		Entzaubern
	-- esES Artículos en la mochila		Desencantar
	-- esMX Artículos en la mochila		Desencantar
	-- frFR Articles dans le sac à dos	Désenchanter
	-- itIT Oggetti nello zaino			Disincantamento
	-- koKR 배낭에있는 물건들			마력 추출
	-- ptBR Itens na mochila			Desencantar
	-- ruRU Предметы в рюкзаке			Распыление
	-- zhCN 背包里的物品					分解
	-- zhTW 背包里的物品					分解
	
	locale = GetLocale();
	
	disenchantName = GetSpellInfo(13262);
	tailoringName = GetSpellInfo(3908);
	leatherworkingName = GetSpellInfo(2108);
	blacksmithingName = GetSpellInfo(2018);
	jewelcraftingName = GetSpellInfo(25229);
	engineeringName = GetSpellInfo(4036);
	inscriptionName = GetSpellInfo(45357);	
	professionsForButton = { tailoringName, leatherworkingName, blacksmithingName, jewelcraftingName, engineeringName, inscriptionName };
	
	_G["BINDING_NAME_CLICK DisenchantCraftedItemsButton:LeftButton"] = disenchantName.." Button";
	
	buttonWidth = 100;
	buttonLocX = -3;
	
	if locale == "enUS" then
		itemCountText = "Items in backpack";
	elseif locale == "deDE" then
		itemCountText = "Gegenstände im Rucksack";
	elseif locale == "esES" then
		itemCountText = "Artículos en la mochila";
	elseif locale == "esMX" then
		itemCountText = "Artículos en la mochila";
	elseif locale == "frFR" then
		itemCountText = "Articles dans le sac à dos";
	elseif locale == "itIT" then
		itemCountText = "Oggetti nello zaino";
		buttonWidth = buttonWidth + 30;
		buttonLocX = buttonLocX - 30;
	elseif locale == "koKR" then
		itemCountText = "배낭에있는 물건들";
	elseif locale == "ptBR" then
		itemCountText = "Itens na mochila";
	elseif locale == "ruRU" then
		itemCountText = "Предметы в рюкзаке";
	elseif locale == "zhCN" then
		itemCountText = "背包里的物品";
	elseif locale == "zhTW" then
		itemCountText = "背包里的物品";
	end
end

local function StringSplit(inputString, separator)
	if not separator then
		separator = "%s";
	end
	
	local splitTable = {}; 
	local splitCounter = 1;
	
	for str in string.gmatch(inputString, "([^"..separator.."]+)") do
		splitTable[splitCounter] = str;
		splitCounter = splitCounter + 1;
	end
	
	return splitTable;
end

local soulboundTooltip = CreateFrame("GameTooltip", "CheckSoulboundTooltip", nil, "GameTooltipTemplate");

local function IsSoulbound(bag, slot)
	soulboundTooltip:SetOwner(UIParent, "ANCHOR_NONE");
	soulboundTooltip:SetBagItem(bag, slot);
	soulboundTooltip:Show();
	
	for i = 1,soulboundTooltip:NumLines() do
		if _G["CheckSoulboundTooltipTextLeft"..i]:GetText()==ITEM_SOULBOUND then
			soulboundTooltip:Hide();
			return true;
		end
	end
	
	soulboundTooltip:Hide()
	return false
end

local function IsDisenchantable(itemID)
	if IsSpellKnown(13262) then
		if itemID then
			local _, _, quality, _, _, _, itemSubType, _, _, _, _, class, subClass = GetItemInfo(tonumber(itemID))
			
			if (class == 2 or class == 4 or (class == 3 and subClass == 11)) and itemSubType ~= "Cosmetic" then
				if quality > 1 and quality < 5 then
					return true;
				end
			end
		end
	end
	
	return false;
end

local function GetBagItemList(selectedItemID)
	itemLocations = {};
	local itemCounter = 0;
	
	for bag = 0, NUM_BAG_SLOTS do
        for slot = 1, C_Container.GetContainerNumSlots(bag) do
			itemID = C_Container.GetContainerItemID(bag, slot)

            if tonumber(itemID) == tonumber(selectedItemID) then
				if not IsSoulbound(bag, slot) then
					if IsDisenchantable(selectedItemID) then
						itemCounter = itemCounter + 1;
						itemLocations[itemCounter] = bag.." "..slot;
					end
				else
					if not ignoreSoulbound then
						if IsDisenchantable(selectedItemID) then
							itemCounter = itemCounter + 1;
							itemLocations[itemCounter] = bag.." "..slot;
						end
					end					
				end
            end
        end
    end
	
	return itemCounter;
end

local button = CreateFrame("Button", "DisenchantCraftedItemsButton", UIParent, "SecureActionButtonTemplate");
local frameItemCount = CreateFrame("Frame", "DisenchantCraftedItemsCount");

local function CreateDCIFrames()
	frameItemCount:SetParent(ProfessionsFrame.CraftingPage.CreateButton);
	frameItemCount:SetSize(250,25);
	frameItemCount:SetPoint("TOPLEFT",ProfessionsFrame.CraftingPage.CreateButton, "TOPLEFT", -209, 23);
	frameItemCount.text = frameItemCount.text or frameItemCount:CreateFontString(nil, "OVERLAY", "GameFontNormal"); --GameFontNormal GameFontNormalLarge GameFontHighlight
	frameItemCount.text:SetAllPoints(true);
	frameItemCount.text:SetJustifyH("LEFT");
	frameItemCount.text:SetJustifyV("TOP");
	frameItemCount.text:SetTextColor(1,0.8046,0,1); --1,0.8046,0,1  1,0.84,0,1
	frameItemCount:Hide();
	
	if IsSpellKnown(13262) then
		button:SetParent(ProfessionsFrame.CraftingPage.CreateButton);
		button:SetSize(buttonWidth, 22); --Width, Height
		button:SetText(disenchantName);
		button:SetPoint("TOPLEFT", ProfessionsFrame.CraftingPage.CreateButton, "TOPLEFT", buttonLocX, 28);
		button:SetNormalFontObject("GameFontNormal");
		button:SetAttribute("type1", "macro");
		button:SetAttribute("macrotext1", "");
		button:Hide();
		
		local ntex = button:CreateTexture();
		ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up");
		ntex:SetTexCoord(0, 0.625, 0, 0.6875);
		ntex:SetAllPoints();
		button:SetNormalTexture(ntex);

		local htex = button:CreateTexture();
		htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight");
		htex:SetTexCoord(0, 0.625, 0, 0.6875);
		htex:SetAllPoints();
		button:SetHighlightTexture(htex);

		local ptex = button:CreateTexture();
		ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down");
		ptex:SetTexCoord(0, 0.625, 0, 0.6875);
		ptex:SetAllPoints();
		button:SetPushedTexture(ptex);
		
		local dtex = button:CreateTexture();
		dtex:SetTexture("Interface/Buttons/UI-Panel-Button-Disabled");
		dtex:SetTexCoord(0, 0.625, 0, 0.6875);
		dtex:SetAllPoints();
		button:SetDisabledTexture(dtex);
	end
end

local function HideDCIFrames()
	locationDisenchantStarted = nil;
	disenchantCasting = false;
	lootOpen = false;
	itemLocked = false;
	
	frameItemCount:Hide();
	button:Hide();
	button:SetAttribute("spell1", nil);
	button:SetAttribute("target-bag1", nil)
	button:SetAttribute("target-slot1", nil)
end

local function UpdateDCIrames(event, ...)
	local openedProfession = ProfessionsFrameTitleText:GetText();		
	local isCraftingProfession = false;
	
	for key, value in pairs(professionsForButton) do 
		if openedProfession and openedProfession:find(value) then
			isCraftingProfession = true;
			break;
		end
	end
	
	if isCraftingProfession then			
		local recipeID = ProfessionsFrame.CraftingPage.SchematicForm.currentRecipeInfo.recipeID;
		
		if recipeID then
			local itemLink = C_TradeSkillUI.GetRecipeItemLink(recipeID);
			
			if itemLink then
				selectedItemID = select(3, strfind(itemLink, "item:(%d+)"))
				
				numberOfItems = GetBagItemList(selectedItemID);
				frameItemCount.text:SetText(itemCountText..": "..numberOfItems);
				
				if previousItemBagSlot ~= itemLocations[1] then
					previousItemBagSlot = itemLocations[1];
					locationDisenchantStarted = nil;
					disenchantCasting = false;
					lootOpen = false;
					itemLocked = false;
				end
				
				if selectedItemID then
					if numberOfItems and numberOfItems > 0 then
						frameItemCount:Show();
						button:Show();
						
						if event == "UNIT_SPELLCAST_START" then
							local unit, spellName, spellID = select(1, ...)
							
							if unit == "player" and spellID == 13262 then
								disenchantCasting = true;
							end
						end
						
						if event == "UNIT_SPELLCAST_SUCCEEDED" then
							local unit, spellName, spellID = select(1, ...)
							
							if unit == "player" and spellID == 13262 then
								locationDisenchantStarted = itemLocations[1];
								
								if autoLoot then
									if GetCVar("autoLootDefault") == "1" then 
										autoLootCurrent = 0;
									else
										autoLootCurrent = 1;
										SetCVar("autoLootDefault", 1);
									end
								end
							end
						end
						
						if event == "UNIT_SPELLCAST_STOP" then
							local unit, spellName, spellID = select(1, ...)
							
							if unit == "player" and spellID == 13262 then
								disenchantCasting = false;									
							end
						end
						
						if event == "UNIT_SPELLCAST_INTERRUPTED" then
							local unit, spellName, spellID = select(1, ...)
							
							if unit == "player" and spellID == 13262 then
								disenchantCasting = false;
							end
						end
						
						if event == "ITEM_LOCKED" then
							local bag, slot = select(1, ...)
							
							if bag and slot then
								if itemLocations[1] == bag.." "..slot then
									itemLocked = true;
								end
							end
						end
						
						if event == "ITEM_UNLOCKED" then
							local bag, slot = select(1, ...)
							
							if bag and slot then
								if itemLocations[1] == bag.." "..slot then
									itemLocked = false;
								end
							else
								itemLocked = false;
							end
						end
						
						if event == "LOOT_OPENED" then
							lootOpen = true;
						end
						
						if event == "LOOT_CLOSED" then
							lootOpen = false;
							
							if autoLoot then
								if autoLootCurrent == 1 then
									SetCVar("autoLootDefault", 0);
								end
							end
						end

						local bagID = nil;
						local slotID = nil;
						
						if not disenchantCasting and not lootOpen and not itemLocked and (not locationDisenchantStarted or locationDisenchantStarted ~= itemLocations[1]) then
							if itemLocations[1] then
								bagID = StringSplit(itemLocations[1], nil)[1];
								slotID = StringSplit(itemLocations[1], nil)[2];
							end
						end
						
						if not disenchantCasting and not lootOpen and not itemLocked and locationDisenchantStarted == itemLocations[1] then
							if itemLocations[2] then
								bagID = StringSplit(itemLocations[2], nil)[1];
								slotID = StringSplit(itemLocations[2], nil)[2];
							end
						end
						
						if itemLocations[1] then
							button:SetAttribute("type1", "spell");
							button:SetAttribute("spell1", disenchantName);
							button:SetAttribute("target-bag1", bagID)
							button:SetAttribute("target-slot1", slotID)
							return;
						end
					end
				end
			end
		end
	end
	
	HideDCIFrames();
end

local f = CreateFrame("Frame", "DisenchantCraftedItemsInitializationFrame")
f:SetScript("OnEvent", function(self, event, ...)
	loadedAddon = select(1, ...);
		
	if loadedAddon == "DisenchantCraftedItems" then
		if not DCI_AutoLootDisenchanted then
			DCI_AutoLootDisenchanted = autoLoot;
		else
			autoLoot = DCI_AutoLootDisenchanted;
		end
		
		if not DCI_IgnoreSoulbound then
			DCI_IgnoreSoulbound = ignoreSoulbound;
		else
			ignoreSoulbound = DCI_IgnoreSoulbound;
		end
		
		SetLocalization();			
	end
end)
f:RegisterEvent("ADDON_LOADED");

frameItemCount:SetScript("OnEvent", function(self, event, ...)
	if event == "TRADE_SKILL_SHOW" then
		CreateDCIFrames();
	end
	
	if ProfessionsFrame:IsShown() then
		UpdateDCIrames(event, ...);
	else
		HideDCIFrames();	
	end	
end)
frameItemCount:RegisterEvent("BAG_UPDATE");
frameItemCount:RegisterEvent("TRADE_SKILL_SHOW");
frameItemCount:RegisterEvent("TRADE_SKILL_LIST_UPDATE");
frameItemCount:RegisterEvent("TRADE_SKILL_DETAILS_UPDATE");
frameItemCount:RegisterEvent("SPELL_DATA_LOAD_RESULT");
frameItemCount:RegisterEvent("UPDATE_TRADESKILL_RECAST");
frameItemCount:RegisterEvent("ITEM_LOCKED");
frameItemCount:RegisterEvent("ITEM_UNLOCKED");
frameItemCount:RegisterUnitEvent("UNIT_SPELLCAST_START", "player");
frameItemCount:RegisterUnitEvent("UNIT_SPELLCAST_SUCCEEDED", "player");
frameItemCount:RegisterUnitEvent("UNIT_SPELLCAST_STOP", "player");
frameItemCount:RegisterUnitEvent("UNIT_SPELLCAST_INTERRUPTED", "player");
frameItemCount:RegisterEvent("LOOT_OPENED");
frameItemCount:RegisterEvent("LOOT_CLOSED");

local f = CreateFrame("Frame", "DisenchantCraftedItemsFalseTaintFixFrame")
f:SetScript("OnEvent", function(self, event, ...)
	local popupText = StaticPopup1 and StaticPopup1.text and StaticPopup1.text:GetText()
	
	if popupText and popupText:find("DisenchantCraftedItems") then
		StaticPopup1.button2:Click()
	end
end)
f:RegisterEvent("ADDON_ACTION_FORBIDDEN");
